<?php
////////////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';
require_once 'lib/utils.php';
require_once 'lib/smart_sdk/sml_api.php';

require_once 'tv_channel.php';
require_once 'tv_group.php';
require_once 'favorites_tv_group.php';
require_once 'all_channels_tv_group.php';

////////////////////////////////////////////////////////////////////////////////

class HomeTv extends AbstractTv
{
	const EXCLUDED_CHANNELS_ATTR_NAME = 'excluded_channels';

	private		$epg_ids = array();	
	private		$SmartAPI;
	
	protected	$mode;
	protected	$drm_prefix;
	protected	$epg_source;

	public	$app_name;
	public	$channels;
	public	$drm_arr = array();
	public	$locations = array();
	public	$regions = array();
	public	$epg_sources = array();

	////////////////////////////////////////////////////////////////////////////

	public function __construct()
	{
		$this->SmartAPI = new SmartAPI;
		parent::__construct(true, PluginConfig::TV_FAVORITES_SUPPORTED, false);
		//$this->SmartAPI = new SmartAPI;
	}

	////////////////////////////////////////////////////////////////////////////

	public function get_ch_nav_mode(&$plugin_cookies)
	{
		return AbstractTv::MODE_CHANNELS_N_TO_M;//(intval(isset($plugin_cookies->ch_nav_mode) ? $plugin_cookies->ch_nav_mode : '1') > 0) ? AbstractTv::MODE_CHANNELS_1_TO_N : AbstractTv::MODE_CHANNELS_N_TO_M;
	}

	public function get_channel($channel_id)
	{
		$ch = $this->channels->get($channel_id);

		if (is_null($ch))
			throw new Exception("Unknown channel: $channel_id");

		return $ch;
	}

	public function get_channel_icon_url($fname)
	{
		return sprintf(PluginConfig::CHANNEL_ICON_URL_FORMAT, $fname);
	}
	
	public function get_group_icon_url($fname)
	{
		return sprintf(PluginConfig::GROUP_ICON_URL_FORMAT, $fname);
	}

	public function get_fav_icon_url(&$plugin_cookies)
	{
		return PluginConfig::FAV_CHANNELS_GROUP_ICON_URL;
	}

	public function get_recycle_bin_icon_url(&$plugin_cookies)
	{
		return PluginConfig::RECYCLE_BIN_GROUP_ICON_URL;
	}

	public function get_exclude_channel_ids(&$plugin_cookies)
	{
		$return_arr = array();

		if (isset($plugin_cookies->{self::EXCLUDED_CHANNELS_ATTR_NAME}))
			$return_arr = preg_split('/,/', $plugin_cookies->{self::EXCLUDED_CHANNELS_ATTR_NAME});

		return $return_arr;
	}

	public function get_plugin_info()
	{
		$doc = file_get_contents('dune_plugin.xml', true);

		if (is_null($doc)) {
			hd_print('Can`t load dune_plugin.xml');
			return 'n/a';
		}

		$xml = HD::parse_xml_document($doc);
		$this->app_name = strval($xml->name);
		$this->app_version = strval($xml->version);
		$this->drm_arr[''] = 'Не использовать';
		foreach ($xml->{'drm_defs'}->children() as $dune_plugin_drm_defs) {
			if ($dune_plugin_drm_defs->getName() == 'drm_def') {
				$value = strval($dune_plugin_drm_defs->id);
				$this->drm_arr[$value] = ucfirst($value);
			}
		}
		//$plugin_cookies->app_name = $this->app_name;
		//$plugin_cookies->app_version = $this->app_version;
	}		
	
	public function folder_entered(MediaURL $media_url, &$plugin_cookies)
	{
		$current_screen = $media_url->get_raw_string();
		//hd_print("folder_entered::MediaURL => ".$current_screen);
		if ($current_screen == TvGroupListScreen::ID)
			$this->load_channels($plugin_cookies);		
	}

	public function get_tv_stream_url($media_url, &$plugin_cookies)
	{		
		return $media_url; 
	}

	public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies)
	{
		$media_url = $this->get_channel($channel_id)->get_streaming_url();

		if (($this->get_channel($channel_id)->is_protected()) &&
		($protect_code !== PluginConfig::PIN_CODE_DEF_VALUE)) {
			$media_url = 'protected';
		}		
		return $media_url;
	}

	////////////////////////////////////////////////////////////////////////////
	
	public function load_channels(&$plugin_cookies) {

		$this->get_plugin_info($plugin_cookies);
		$this->drm_prefix = isset($plugin_cookies->use_drm) ? $plugin_cookies->use_drm : '';
		$this->epg_source = isset($plugin_cookies->epg_source) ? $plugin_cookies->epg_source : PluginConfig::EPG_SOURCE_DEF_VALUE;
		$use_proxy = isset($plugin_cookies->use_proxy) ? $plugin_cookies->use_proxy : PluginConfig::USE_PROXY_DEF_VALUE;
		$buffering_time = isset($plugin_cookies->buffering_ms) ? $plugin_cookies->buffering_ms : PluginConfig::BUFFERING_TIME_DEF_VALUE;		
		$exclude_channel_ids = $this->get_exclude_channel_ids($plugin_cookies);

		$after_install_idx = isset($plugin_cookies->after_install_idx) ? intval($plugin_cookies->after_install_idx) : 0;
		if ($after_install_idx < 3) {
			$plugin_cookies->macro_id = '1';
			$plugin_cookies->region_id = PluginConfig::REGION_ID_DEF_VALUE;
			$plugin_cookies->after_install_idx = '3';
		}

		$region_id = isset($plugin_cookies->region_id) ? intval($plugin_cookies->region_id) : PluginConfig::REGION_ID_DEF_VALUE;
		//$this->time_zone = isset($plugin_cookies->time_zone) ? $plugin_cookies->time_zone : PluginConfig::EPG_TIME_ZONE_DEF_VALUE;
		
		//$this->SmartAPI = new SmartAPI;
		
		$doc = file_get_contents(PluginConfig::CHANNEL_LIST_URL, true);

		if (is_null($doc))
			throw new Exception(T::t('msg_can_not_fetch_playlist'));

		$xml = HD::parse_xml_document($doc);

		if ($xml->getName() !== 'tv_info')
			throw new Exception(T::t('msg_invalid_xml_document'));

		foreach ($xml->{'tv_locations'}->children() as $xml_tv_location) {
			$this->locations[intval($xml_tv_location->id)] = strval($xml_tv_location->caption);
			
			foreach ($xml_tv_location->{'sub'}->{'location'} as $xml_tv_region) {
				$this->regions[intval($xml_tv_location->id)][intval($xml_tv_region->id)] = strval($xml_tv_region->caption);

				if (intval($xml_tv_region->id) == $region_id) {
					//if ($this->time_zone == 'auto') {
					//	$this->time_zone = strval($xml_tv_region->time_zone);
					//}
					
					$this->SmartAPI->set_location(intval($xml_tv_region->id));
				}
			}
		}	

		$this->channels = new HashedArray();
		$this->groups = new HashedArray();

		$this->mode = $this->get_ch_nav_mode($plugin_cookies);		

		if ($this->is_favorites_supported())
			$this->groups->put(
				new FavoritesTvGroup(
					$this,
					T::t('title_fav_channels_group'),
					$this->get_fav_icon_url($plugin_cookies)));

		$this->groups->put(
			new AllChannelsTvGroup(
				$this,
				T::t('title_all_channels_group'),
				PluginConfig::ALL_CHANNELS_GROUP_ICON_URL));

		foreach ($xml->{'tv_categories'}->children() as $xml_tv_group)
		{
			$this->groups->put(
				new TvGroup(
					$this,
					strval($xml_tv_group->id),
					strval($xml_tv_group->caption),
					$this->get_group_icon_url(strval($xml_tv_group->logo))));
		}

		if ($xml->{'tv_channels'}->count() == 0)
			throw new Exception(T::t('msg_did_not_match_any_channel'));

		//$this->epg_sources[0] = 'Автовыбор (S->V->A)';
		foreach ($xml->{'tv_channels'}->{'channel'} as $xml_channel)
		{
			//$url = '';
			$cid = strval($xml_channel->id);
			$protected = false;
			$location_is_found = false;
			//if ($xml_channel->getName() == 'channel')
				//foreach ($xml_channel->children() as $xml_ch_streams)
				foreach ($xml_channel->{'streams'}->{'stream'} as $xml_ch_streams_region)
				{
					//if ($xml_ch_streams->getName() == 'streams')
						//foreach ($xml_ch_streams->children() as $xml_ch_streams_region)
						//foreach ($xml_ch_streams->{'stream'} as $xml_ch_streams_region)
						//{
							foreach ($xml_ch_streams_region->{'locations'}->{'id'} as $xml_ch_streams_location_id) {
							//if ($xml_ch_streams_region->getName() == 'location')
								//if (strval(trim($xml_ch_streams_region->url)) !== '') {
									$url = strval(trim($xml_ch_streams_region->url));
									//$reg_array = explode(',', $xml_ch_streams_region->ids);
									
									//foreach ($reg_array as $rid) {
									//foreach ($xml_ch_streams_location->id as $rid) {
										if (intval($xml_ch_streams_location_id) === $region_id) {
											$epg_timeshift = $xml_ch_streams_region->time_shift;
											$url = trim($xml_ch_streams_region->url);
											$protected = (intval($xml_ch_streams_region->protected) > 0);
											foreach ($xml_ch_streams_region->{'epg'}->children() as $xml_ch_streams_stream_epg) {
												$epg_src = strval($xml_ch_streams_stream_epg->getName());
												$this->epg_ids[$epg_src][$cid] = strval($xml_ch_streams_stream_epg);
												//hd_print($epg_src.'='.strval($xml_ch_streams_stream_epg).'for cid='.$cid);
											}
											$location_is_found = true;
											break;
										}
									//}
								}
							//if (!empty($url)) break;
							//if ($location_is_found) break;
						//}
					//if (!empty($url)) break;
					if ($location_is_found) break;
				}

			if (!$location_is_found) continue;

			$url = 'udp://@' . $url;

			if ($use_proxy == 'yes')
				$url = str_replace('udp://@', 'http://ts://'.$plugin_cookies->proxy_ip_port.'/udp/', $url);

			if ($protected && ($this->drm_prefix <> '')) {
				$url = 'drm://' . $this->app_name . '/' . $this->drm_prefix . '/' . $url;
			}

			//$cid = strval($xml_channel->id);

			//$xml_ch_epg = $xml_channel->{'epg'}->children();
			//$this->epg_ids[$cid]['sml'] = empty($xml_ch_epg->sml)? '' : strval($xml_ch_epg->sml);
			//$this->epg_ids[$cid]['akado'] = empty($xml_ch_epg->akado)? '' : strval($xml_ch_epg->akado);
			//$this->epg_ids[$cid]['vsetv'] = empty($xml_ch_epg->vsetv)? '' : strval($xml_ch_epg->vsetv);

			$channel =
				new TvChannel(
					$cid,
					intval($xml_channel->number),
					strval(trim($xml_channel->caption)),
					strval(trim($xml_channel->description)),
					strval(trim($xml_channel->language)),
					$this->get_channel_icon_url(trim(strval($xml_channel->logo))),
					$url,
					intval($xml_channel->adult),
					PluginConfig::NUM_PAST_EPG_DAYS,
					PluginConfig::NUM_FUTURE_EPG_DAYS,
					intval($buffering_time),
					intval($epg_timeshift));

			$this->channels->put($channel);

			//$this->mode = $this->get_ch_nav_mode($plugin_cookies);

			if ($xml_channel->{'categories'}->count() == 0)
				continue;

			//$xml_ch_category_id = $xml_channel->{'categories'}->children();
				//if ($xml_ch_category_id->getName() == 'id')
				//{
					//$tv_cat_array = explode(",", strval($xml_ch_category_id));
					foreach ($xml_channel->{'categories'}->{'id'} as $gid) {
					//foreach ($xml_ch_category_id as $gid) {
						$group = $this->groups->get(intval($gid));
						$channel->add_group($group->get_id());
						if ($this->mode == AbstractTv::MODE_CHANNELS_1_TO_N)
							break;
					}
				//}
		}
		$this->epg_sources = array_merge(array('auto' => 'Автовыбор'), array_keys($this->epg_ids));
		TvChannelsScreen::$ch_sorter->sort_channels($plugin_cookies);
	}

	///////////////////////////////////////////////////////////////////////////	

	private function get_day_epg_akado($cid, $date) {
		
		$epg_id = empty($this->epg_ids['AKADO'][$cid])? null : $this->epg_ids['AKADO'][$cid];
		if (empty($epg_id)) return false;
		
		$epg_date = date("Y-m-d", $date);
		try {
			$html = HD::http_get_document(sprintf(PluginConfig::AKADO_URL_FORMAT, $epg_id, $epg_date));
		}
		catch (Exception $e) {
			hd_print("Can't fetch from AKADO (ID:$epg_id DATE:$epg_date)");
			return false;
		}

		$patterns = array(
		"/\n/", "/.*<table class=\"tv-channel-full\">/", "/<\/table><\/div>.*/", "/<tr.*?tv-time.>/", "/<\/[ap].*?p>/", "/<\/span>/", "/[0-9][0-9]\+ /");
		$replace = array("", "", "", "\n", "|", "|", "");

		$html = strip_tags(preg_replace($patterns, $replace, $html));

		$html = preg_replace_callback(
			"/[0-2][0-9]:[0-5][0-9]/", 
			create_function(
				'$matches',
				'return strtotime("'.$epg_date.' $matches[0] GMT+4") . "-0";'),
			$html);

		$epg_items = explode("\n", trim($html, "\n"));

		if (empty($epg_items)) return false;

		return $epg_items;
	}

	private function get_day_epg_vsetv($cid, $date) {

		$epg_id = empty($this->epg_ids['VSETV'][$cid])? null : $this->epg_ids['VSETV'][$cid];
		if (empty($epg_id)) return false;
		
		$epg_date = date("Y-m-d", $date);
		try {
			$html = HD::http_get_document(sprintf(PluginConfig::VSETV_URL_FORMAT, $epg_id, $epg_date));//, $time_zone_vals[$this->time_zone]));
			$html = iconv('WINDOWS-1251', 'UTF-8', $html);
		}
		catch (Exception $e) {
			hd_print("Can't fetch from VSETV (ID=$epg_id DATE=$epg_date)");
			return false;
		}

		$patterns = array("/\n/", "/.*?<div class=\"clear\">.*?<div id=\"schedule_container\".*?>/", "/<div class=\"clear\">.*/", "/<div class=\"pasttime\">/", "/<div class=\"onair\">/", "/<div class=\"time\">/", "/<div class=\"pastprname\">|<div class=\"prname\">/", "/<div class=\"desc\">/", "/<br><br>/", "/<br>/", "/&nbsp;/");
		$replace = array("", "", "", "\n", "\n", "\n", "|", "|", ". ", ". ", "");

		$html = strip_tags(preg_replace($patterns, $replace, $html));
		//hd_print($html);
		$html = preg_replace_callback(
			"/[0-2][0-9]:[0-5][0-9]/", 
			create_function(
				'$matches',
				'return strtotime("'.$epg_date.' $matches[0] GMT+4");'),//'.$this->time_zone.'");'),
			$html);

		$epg_items = explode("\n", trim($html, "\n"));

		if (empty($epg_items)) return false;

		return $epg_items;
	}
	
	public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
	{	
		$source_is_html = false;
		if ($this->epg_source == 'auto') {
			$epg_id = empty($this->epg_ids['SMARTLABS'][$channel_id])? '' : $this->epg_ids['SMARTLABS'][$channel_id];
			$epg_arr = $this->SmartAPI->get_day_epg($epg_id, $day_start_ts);
			if (empty($epg_arr)) {
				$source_is_html = true;
				$epg_arr = $this->get_day_epg_vsetv($channel_id, $day_start_ts);
				if (empty($epg_arr)) {
					$epg_arr = $this->get_day_epg_akado($channel_id, $day_start_ts);
				}
			}			
		} else if ($this->epg_sources[$this->epg_source] == 'SMARTLABS') {
					$epg_id = empty($this->epg_ids['SMARTLABS'][$channel_id])? '' : $this->epg_ids['SMARTLABS'][$channel_id];
					$epg_arr = $this->SmartAPI->get_day_epg($epg_id, $day_start_ts);
		} else if ($this->epg_sources[$this->epg_source] == 'VSETV') {
					$epg_arr = $this->get_day_epg_vsetv($channel_id, $day_start_ts);
					$source_is_html = true;
		} else if ($this->epg_sources[$this->epg_source] == 'AKADO') {
					$epg_arr = $this->get_day_epg_akado($channel_id, $day_start_ts);
					$source_is_html = true;
		} 
		
		if (empty($epg_arr)) return false;
		
		$day_end_ts = $day_start_ts + 86400 + 14400;
		$start_ts = 0;
		$stop_ts = 0;
		foreach ($epg_arr as $item) {
			$epg_items = explode('|', $item);
			$items_count = count($epg_items);
			$epg_ts = ($items_count > 0)? explode('-', $epg_items[0]) : null;
			$item_start_utime = intval($epg_ts[0]);
			$item_stop_utime = intval($epg_ts[1]);
			if ($source_is_html) {
				$start_ts = intval(($item_start_utime < $start_ts) ? $item_start_utime + 86400 : $item_start_utime);
				if ($item_stop_utime > 0) {
					$stop_ts = intval(($item_stop_utime < $stop_ts) ? $item_stop_utime + 86400 : $item_stop_utime);					
				} else
					$stop_ts = -1;
			} else {
				$start_ts = $item_start_utime;
				$stop_ts = $item_stop_utime;
				/*if (($start_ts < $day_start_ts) || ($start_ts >= $day_end_ts)) {
					hd_print('-['.$epg_items[0].']: '.$epg_items[1]);
					continue;
				}*/				
			}
			$epg[$start_ts]['name'] = ($items_count > 1)? $epg_items[1] : '';
			$epg[$start_ts]['desc'] = ($items_count > 2)? $epg_items[2] : '';
			$epg[$start_ts]['limit'] = $stop_ts;
			//hd_print('++['.$start_ts.'-'.$stop_ts.']: '.$epg[$start_ts]['name'] );			
		}
		
		$timeshift_hours = $this->channels->get($channel_id)->get_timeshift_hours();
		$timeshift_sec = $timeshift_hours*3600;
		
		$epg_result = array();
		if (isset($epg)) {
			foreach ($epg as $time => $value) {
				$epg_result[] =
						new DefaultEpgItem(
							strval($value['name']),
							strval($value['desc']),
							intval($time + $timeshift_sec),
							intval($value['limit'] + $timeshift_sec));
			}

			return
				new EpgIterator(
					$epg_result,
					$day_start_ts,
					$day_start_ts + 100400);	
		} else 
			return false;
	}

	///////////////////////////////////////////////////////////////////////

	public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
	{
		$this->mode = $this->get_ch_nav_mode($plugin_cookies);

		$this->ensure_channels_loaded($plugin_cookies);

		$groups = array();
		$channels = array();

		foreach ($this->get_groups() as $g)
		{
			if ($g->is_favorite_channels())
				continue;
			$group_channels_arr = $g->get_channels($plugin_cookies);

			if (count($group_channels_arr) == 0)
				continue;

			if ($g->is_all_channels())
			{
				foreach ($group_channels_arr as $cid)
				{
					$group_id_arr = array();

					if ($this->mode == self::MODE_CHANNELS_N_TO_M)
						$group_id_arr[] = $this->get_all_channel_group_id();

					$c = $this->get_channel($cid);

					array_push($channels,
						array
						(
							PluginTvChannel::id => $c->get_id(),
							PluginTvChannel::caption => $c->get_title(),
							PluginTvChannel::group_ids => array_merge($group_id_arr, $c->get_groups()),
							PluginTvChannel::icon_url => $c->get_icon_url(),
							PluginTvChannel::number => $c->get_number(),

							PluginTvChannel::have_archive => $c->has_archive(),
							PluginTvChannel::is_protected => $c->is_protected(),

							PluginTvChannel::past_epg_days => intval($c->get_past_epg_days()),
							PluginTvChannel::future_epg_days => intval($c->get_future_epg_days()),

							PluginTvChannel::archive_past_sec => intval($c->get_archive_past_sec()),
							PluginTvChannel::archive_delay_sec => intval($c->get_archive_delay_sec()),

							PluginTvChannel::buffering_ms => intval($c->get_buffering_ms()),
							PluginTvChannel::timeshift_hours => intval($c->get_timeshift_hours()),

							PluginTvChannel::playback_url_is_stream_url => false,
						));
				}

				if ($this->mode == self::MODE_CHANNELS_1_TO_N)
					continue;
			}

			array_push($groups,
				array
				(
					PluginTvGroup::id => $g->get_id(),
					PluginTvGroup::caption => $g->get_title(),
					PluginTvGroup::icon_url => $g->get_icon_url()
				));
		}

		$is_favorite_group = isset($media_url->is_favorites);
		$initial_group_id = strval($media_url->group_id);
		$initial_is_favorite = 0;

		if ($is_favorite_group)
		{
			$initial_group_id = null;
			$initial_is_favorite = 1;
		}

		if ($this->mode == self::MODE_CHANNELS_1_TO_N &&
			$initial_group_id === $this->get_all_channel_group_id())
		{
			$initial_group_id = null;
		}

		$fav_channel_ids = null;
		if ($this->is_favorites_supported())
			$fav_channel_ids = $this->get_fav_channel_ids($plugin_cookies);

		$archive = $this->get_archive($media_url, $plugin_cookies);
		$archive_def = is_null($archive) ? null : $archive->get_archive_def();

		return array
		(
			PluginTvInfo::show_group_channels_only => $this->mode,

			PluginTvInfo::groups => $groups,
			PluginTvInfo::channels => $channels,

			PluginTvInfo::favorites_supported => $this->is_favorites_supported(),
			PluginTvInfo::favorites_icon_url => strval($this->get_fav_icon_url($plugin_cookies)),

			PluginTvInfo::initial_channel_id => strval($media_url->channel_id),
			PluginTvInfo::initial_group_id => $initial_group_id,

			PluginTvInfo::initial_is_favorite => $initial_is_favorite,
			PluginTvInfo::favorite_channel_ids => $fav_channel_ids,

			PluginTvInfo::archive => $archive_def,
		);
	}

	///////////////////////////////////////////////////////////////////////

	public function add_special_groups(&$items, &$plugin_cookies)
	{
		$items[] = array
		(
			PluginRegularFolderItem::media_url => TvRecycleBinScreen::get_media_url_str(),
			PluginRegularFolderItem::caption => T::t('title_recycle_bin'),
			PluginRegularFolderItem::view_item_params => array
			(
				ViewItemParams::icon_path => $this->get_recycle_bin_icon_url($plugin_cookies),
				ViewItemParams::item_detailed_icon_path => $this->get_recycle_bin_icon_url($plugin_cookies),
				ViewItemParams::item_caption_color => 21,
			)
		);
	}

	///////////////////////////////////////////////////////////////////////

	public function get_tv_group_list_folder_views(&$plugin_cookies)
	{
		return PluginConfig::GET_TV_GROUP_LIST_FOLDER_VIEWS();
	}

	public function get_tv_channel_list_folder_views(&$plugin_cookies)
	{
		return PluginConfig::GET_TV_CHANNEL_LIST_FOLDER_VIEWS();
	}

}

////////////////////////////////////////////////////////////////////////////////
?>
