<?php

// Root URI
define("API_SERVER_ROOT_URI",		"http://sdp.svc.iptv.rt.ru:8080/");
define("API_SERVER_ROOT_URI_S",		"https://sdp.svc.iptv.rt.ru:8443/");
define("BACKUP_DHCP_SETTIGS", 		"network.xml");

// Cache
define("SIMPLECACHE_URI",			"CacheClient/simple/ncdxml/");
define("CACHE_URI",					"CacheClient/ncdxml/");

// Lists
define("ACCESS_LEVEL_DICT_URI",		"simplecache:AccessLevelDictionary/list");
define("COUNTRES_DICT_URI",			"simplecache:CountryDictionary/list");
define("CHANNEL_SUBJECT_URI",		"simplecache:ChannelSubject/list"); // Категории каналов
define("PROGRAM_GENRE_URI",			"simplecache:ProgramGenre/list");
define("LIST_SERVICES_RT_URI",		"simplecache:ProductOffer/list_services_rt?locationId=%s&productOfferId=1735650");
define("LIST_CHANNEL_PACKAGE_URI",	"cache:ChannelPackage/list_channels?channelPackageId=%s&locationId=%s&from=0&to=2147483647");
define("CHANNEL_EPG_URI",			"cache:EPG/get_by_Chnnl?channelId=%s&day=%s&locationId=%s&from=0&to=99999");


class SmartAPI
{	
	private $access_levels = array();
	private $countres = array();
	private $categores_list = array();
	private $program_genres = array();

	private	$_location_id = 0;		

	protected $init_status = 0;

	public function __construct() {
		$this->init_status = $this->smart_api_init();
	}

	protected function get_xml_from_uri($URI) {
		
		if (empty($URI)) return false;

		$url = API_SERVER_ROOT_URI;
		$url .= preg_replace(array('/simplecache:/', '/cache:/'), array(SIMPLECACHE_URI, CACHE_URI), $URI);
		try {
			$doc = HD::http_get_document($url);
			$xml = HD::parse_xml_document($doc);
		}
		catch (Exception $e) {
			hd_print("Can't fetch from URI: '$url'");
			return false;
		}
		
		return $xml; 		
	}
	
	protected function smart_api_init() {
		return (//$this->load_countres_list(COUNTRES_DICT_URI) &&
				$this->load_program_genres_list(PROGRAM_GENRE_URI) &&
				//$this->load_categores_list(CHANNEL_SUBJECT_URI) &&
				$this->load_access_levels_list(ACCESS_LEVEL_DICT_URI));
	}

	protected function load_access_levels_list($url) {
		
		if (!$xml = $this->get_xml_from_uri($url)) return false;

		foreach ($xml->{'row'} as $row) {
			$this->access_levels[intval($row->AccessLevelDictionary_ID)] = strval($row->AccessLevelDictionary_name);
		}
		return (empty($this->access_levels)? false : true);
	}
	
	protected function load_categores_list($url) {
		
		if (!$xml = $this->get_xml_from_uri($url)) return false;

		foreach ($xml->{'row'} as $row) {
			$this->categores_list[intval($row->ChannelSubject_ID)] = preg_replace(array('/ой/', '/й/'), array('ые', 'е'), strval($row->ChannelSubject_name));
		}
		return (empty($this->categores_list)? false : true);
	}
	
	protected function load_countres_list($url) {
		
		if (!$xml = $this->get_xml_from_uri($url)) return false;

		foreach ($xml->{'row'} as $row) {
			$this->countres[intval($row->CountryDictionary_ID)] = strval($row->CountryDictionary_name);
		}
		return (empty($this->countres)? false : true);
	}

	protected function load_program_genres_list($url) {
		
		if (!$xml = $this->get_xml_from_uri($url)) return false;

		foreach ($xml->{'row'} as $row) {
			$this->program_genres[intval($row->ProgramGenre_ID)] = strval($row->ProgramGenre_name);
		}
		return (empty($this->program_genres)? false : true);
	}

	public function get_status() {
		return $this->init_status;
	}

	public function get_location() {
		return $this->_location_id;
	}	

	public function set_location($location_id) {
		$this->_location_id = $location_id;
	}
	
	public function get_access_level($level_id) {
		return @$this->access_levels[$level_id];
	}

	public function get_categores_list() {
		return $this->categores_list;
	}
	
	public function get_country($country_id) {
		return @$this->countres[$country_id];
	}

	public function get_program_genre($genre_id) {
		return @$this->program_genres[$genre_id];
	}	

	public function get_day_epg($channel_id, $day_start_ts) {
		//return false;
		//hd_print("day_start_ts=$day_start_ts");
		$location = $this->get_location();

		if (empty($channel_id) ||
			empty($location) ||
			empty($this->init_status)
			) return false;
		
		//hd_print("day_start_ts=$day_start_ts");
		
		$epg_items = array();		
		$day_ts = $day_start_ts;
		$last_start_ts = $day_ts;
		
		while ($day_ts <= $day_start_ts + 86400) {
			//hd_print("day_ts=$day_ts");
			$epg_date = date("m/d/Y", $day_ts);

			if (!$xml = $this->get_xml_from_uri(sprintf(CHANNEL_EPG_URI, $channel_id, $epg_date, $location)))
				return false;
			
			foreach($xml->{'epg'}->children() as $xml_items) {
				$item_start_ts = intval($xml_items->sdate);
				$item_finish_ts = intval($xml_items->fdate);
				
				// Если элемент из предыдущего дня, то пропускаем его
				//if (($item_start_ts <= $last_start_ts) ||
				//	($item_start_ts < $day_ts)) continue;

				$al = $this->get_access_level(intval($xml_items->al));
				//$country = (isset($xml_items->country)) ? $this->get_country(intval($xml_items->country)) : '';
				$year = isset($xml_items->year) ? strval($xml_items->year) : '';
				$genre = isset($xml_items->genre) ? $this->get_program_genre(intval($xml_items->genre)) : '';
				$director = isset($xml_items->director) ? strval($xml_items->director) : '';
				$actors = isset($xml_items->actors) ? strval($xml_items->actors) : '';
				$leading = isset($xml_items->leading) ? strval($xml_items->leading) : '';

				$desc = '';
				// Возрастное ограничение: [Х+]
				if (!empty($al)) $desc .= '[' . $al . '] ';				
				// Страна-производитель контента
				//if (!empty($country)) $desc .= $country . ', ';
				// Год производства
				if (!empty($year)) $desc .= $year . ', ';
				// Жанр
				if (!empty($genre)) $desc .= strtolower($genre) . ', ';
				// Если указан директор, 
				if (!empty($director)) {
					// и если указаны актеры, 
					if (!empty($actors)) {
						// то директор является режиссером, включаем его с актерами в описание.
						$desc .= 'реж. ' . $director . ', актеры: ' . $actors;
					} else // иначе (если актеры не указаны), 
						// то директор является автором, и так же пишем в описании
						$desc .= 'авт. ' . $director;
				} else { // Иначе (если директор не указан)
					// Если указаны актеры, то они являются ведущими, и так же пишем в описании
					$desc .= (!empty($actors)) ? 'актеры: ' . $actors : '';
					// Если указан ведущий, то так и пишем в описании
					$desc .= (!empty($leading)) ? 'вед. ' . $leading : '';
				}
				
				$desc = trim($desc);
				// Далее идет краткая аннотация передачи
				$desc = rtrim($desc, ',') . '. ' . trim(strval($xml_items->desc)) . '...';

				$last_start_ts = $item_start_ts;
				// Собираем все в строку вида {StartUTS-StopUTS}|{Название}|{Описание}, и помещаем в итоговый массив
				$epg_items[] = $item_start_ts . 
						'-' . $item_finish_ts .
						// Из названия также удаляем возрастное ограничение [Х+]
						'|' . rtrim(preg_replace('/\[.*\]/', '', $xml_items->name)) .
						// А из описания удаляем переносы строк
						'|' . preg_replace("/\n/", ' ', $desc);
				//if (($item_start_ts <= $last_start_ts) ||
				//	($item_start_ts < $day_ts)) {
				//		hd_print('-'.array_pop($epg_items));
				//	} else hd_print('+'.$epg_items[count($epg_items)-1]);
			}
			// инкремент даты на сутки
			$day_ts += 86400;
			// принудительная пауза между запросами на сервер
			usleep(200000);
		}
		
		return $epg_items;
	}
	
}
	
?>
