<?php
////////////////////////////////////////////////////////////////////////////////

require_once 'lib/user_input_handler.php';
require_once 'lib/control_factory.php';
require_once 'lib/action_factory.php';
require_once 'lib/utils.php';

////////////////////////////////////////////////////////////////////////////////

class SettingsDialog implements UserInputHandler
{

	private static $instance = null;
	

	public static function init($owner)
	{
		if (is_null(self::$instance))
			self::$instance = new SettingsDialog($owner);
		return self::$instance;
	}

	////////////////////////////////////////////////////////////////////////////

	const ID = 'settings_dialog';

	private $time_zone;
	private $mrid;
	private $rid;
	private $use_drm;
	private $use_proxy;
	private $proxy_ip_port;
	private $buffering_time;
	private $epg_source;
	//private $ch_nav_mode;

	protected $_owner;
	
	public	$tv;
	
	////////////////////////////////////////////////////////////////////////////

	public function __construct($owner)
	{
		$this->_owner = $owner;
		UserInputHandlerRegistry::get_instance()->register_handler($this);
	}

	public function get_handler_id()
	{
		return self::ID.'_handler';
	}
	
	public function handle_user_input(&$user_input, &$plugin_cookies)
	{
		/* ---------- BEGIN DEBUG ---------- */
		//hd_print('SettingsDialog $user_input =');
		//foreach ($user_input as $key => $value)
			//hd_print("  $key => $value");
		/* ----------- END DEBUG ----------- */

		$control_id = $user_input->control_id;

		if ($user_input->action_type == 'confirm')
			{
				$new_value = $user_input->{$control_id};
				if ($control_id == 'time_zone')
					$this->time_zone = $new_value;
				else if ($control_id == 'mrid')
						$this->mrid = $new_value;						
				else if ($control_id == 'rid')
						$this->rid = $new_value;
				else if ($control_id == 'use_drm')
						$this->use_drm = $new_value;						
				else if ($control_id == 'use_proxy')
						$this->use_proxy = $new_value;
				else if ($control_id == 'proxy_ip_port')
						$this->proxy_ip_port = $new_value;
				else if ($control_id == 'buffering_time')
						$this->buffering_time = $new_value;
				else if ($control_id == 'epg_source')
						$this->epg_source = $new_value;
				//else if ($control_id == 'ch_nav_mode')
				//        $this->ch_nav_mode = $new_value;
			}
		else if ($control_id == 'apply_btn')
			{
				$plugin_cookies->time_zone = $this->time_zone;
				$plugin_cookies->macro_id = $this->mrid;
				$plugin_cookies->region_id = $this->rid;
				$plugin_cookies->use_drm = $this->use_drm;
				$plugin_cookies->use_proxy = $this->use_proxy;
				$plugin_cookies->proxy_ip_port = $this->proxy_ip_port;
				$plugin_cookies->buffering_ms = $this->buffering_time;
				$plugin_cookies->epg_source = $this->epg_source;
				//$plugin_cookies->ch_nav_mode = $this->ch_nav_mode;

				$parent_media_url = MediaURL::decode($user_input->parent_media_url);
				$range = $this->_owner->get_folder_range($parent_media_url, 0, $plugin_cookies);
				return ActionFactory::update_regular_folder($range, true, $user_input->sel_ndx);
			}

		return ActionFactory::reset_controls($this->do_get_controls_defs($plugin_cookies));
	}

	public function do_get_controls_defs(&$plugin_cookies)
	{
		$defs  = array();
		ControlFactory::add_vgap($defs, -10);
		ControlFactory::add_label($defs, T::t('cap_plugin_version'), $this->tv->app_version);
		ControlFactory::add_vgap($defs, -10);
		ControlFactory::add_label($defs, T::t('cap_plugin_copyright'), PluginConfig::PLUGIN_COPYRIGHT_INFO);
		ControlFactory::add_combobox($defs, $this, null, 'mrid', 'Макрорегион:', $this->mrid, $this->tv->locations, 0, true);
		ControlFactory::add_combobox($defs, $this, null, 'rid', T::t('cap_region'), $this->rid, $this->tv->regions[$this->mrid], 0, true);
		ControlFactory::add_combobox($defs, $this, null, 'epg_source', 'Источник EPG:', $this->epg_source, $this->tv->epg_sources, 0, true);
		//ControlFactory::add_combobox($defs, $this, null, 'time_zone', T::t('cap_epg_timeshift'), $this->time_zone, PluginConfig::$time_zone_ops, 0, true);
		ControlFactory::add_combobox($defs, $this, null, 'use_drm', 'Verimatrix DRM', $this->use_drm, $this->tv->drm_arr, 0, true);
		ControlFactory::add_combobox($defs, $this, null, 'use_proxy', T::t('cap_use_proxy'), $this->use_proxy, PluginConfig::$use_proxy_ops, 0, true);
		if ($this->use_proxy == 'yes') {
			ControlFactory::add_text_field($defs, $this, null, 'proxy_ip_port', T::t('cap_proxy_ip_port'), $this->proxy_ip_port, false, false, false, true, 500, true, false);
			ControlFactory::add_combobox($defs, $this, null, 'buffering_time', T::t('cap_buffering_time'), $this->buffering_time, PluginConfig::$buffering_ops, 0, true);
		}
		//ControlFactory::add_combobox($defs, $this, null, 'ch_nav_mode', T::t('cap_navigation_mode'), $this->ch_nav_mode, PluginConfig::$ch_nav_mode_ops, 0, true);
		ControlFactory::add_vgap($defs, 40);

		$add_params = array('button_caption_centered' => 1);
		ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, 'apply_btn', T::t('cap_btn_apply'), 300);
		ControlFactory::add_close_dialog_button($defs, T::t('cap_btn_cancel'), 300, $add_params);
		return $defs;
	}

	public function show_settings_dialog(&$plugin_cookies)
	{
		//$this->time_zone = isset($plugin_cookies->time_zone) ? $plugin_cookies->time_zone : PluginConfig::EPG_TIME_ZONE_DEF_VALUE;
		$this->mrid = isset($plugin_cookies->macro_id) ? intval($plugin_cookies->macro_id) : 1;
		$this->rid = isset($plugin_cookies->region_id) ? intval($plugin_cookies->region_id) : PluginConfig::REGION_ID_DEF_VALUE;
		$this->use_drm = isset($plugin_cookies->use_drm) ? $plugin_cookies->use_drm : '';
		$this->use_proxy = isset($plugin_cookies->use_proxy) ? $plugin_cookies->use_proxy : PluginConfig::USE_PROXY_DEF_VALUE;
		$this->proxy_ip_port = isset($plugin_cookies->proxy_ip_port) ? $plugin_cookies->proxy_ip_port : PluginConfig::PROXY_IP_PORT_DEF_VALUE;
		$this->buffering_time = isset($plugin_cookies->buffering_ms) ? $plugin_cookies->buffering_ms : PluginConfig::BUFFERING_TIME_DEF_VALUE;
		$this->epg_source = isset($plugin_cookies->epg_source) ? $plugin_cookies->epg_source : PluginConfig::EPG_SOURCE_DEF_VALUE;
		//$this->ch_nav_mode = isset($plugin_cookies->ch_nav_mode) ? $plugin_cookies->ch_nav_mode : PluginConfig::CHANNEL_NAV_DEF_VALUE;

		return ActionFactory::show_dialog(T::t('title_settings'),
			$this->do_get_controls_defs($plugin_cookies), false);
	}
}

///////////////////////////////////////////////////////////////////////////
?>
