<?php
////////////////////////////////////////////////////////////////////////////////

require_once 'lib/user_input_handler.php';
require_once 'lib/control_factory.php';
require_once 'lib/action_factory.php';

////////////////////////////////////////////////////////////////////////////////

class SortingDialog implements UserInputHandler
{

    private static $instance = null;

    public static function init($owner)
    {
        if (is_null(self::$instance))
            self::$instance = new SortingDialog($owner);
        return self::$instance;
    }

    ///////////////////////////////////////////////////////////////////////
	// Sort functions

    public static function sort_ch_by_number($a, $b)
    {
        return strnatcasecmp($a->get_number(), $b->get_number());
    }
    
    public static function sort_ch_by_title($a, $b)
    {
        return strnatcasecmp($a->get_title(), $b->get_title());
    }

    public static function sort_ch_by_number_priority_hd($a, $b)
    {
		$_a = intval($a->get_number());
		$_b = intval($b->get_number());
		if (($_a >= 900)&&($_b >= 900)): return strnatcasecmp($_a, $_b);
            elseif ($_a >= 900): return -1;
                elseif ($_b >= 900): return 1;
                    else: return strnatcasecmp($_a, $_b);
        endif;
    }    

    public static function sort_ch_by_title_priority_hd($a, $b)
    {
		$_a = intval($a->get_number());
		$_b = intval($b->get_number());
		if (($_a >= 900)&&($_b >= 900)): return strnatcasecmp($a->get_title(), $b->get_title());
            elseif ($_a >= 900): return -1;
                elseif ($_b >= 900): return 1;
                    else: return strnatcasecmp($a->get_title(), $b->get_title());
        endif;
    }
    
	////////////////////////////////////////////////////////////////////////////

	const ID = 'sorting_dialog';
	const SORTING_METHOD_ATTR_NAME = 'sorting_method_idx';

    const SORT_CH_BY_NUMBER_FUNC    = 'SortingDialog::sort_ch_by_number';
    const SORT_CH_BY_TITLE_FUNC     = 'SortingDialog::sort_ch_by_title';
    const SORT_CH_BY_NUMBER_PRIORITY_HD_FUNC   = 'SortingDialog::sort_ch_by_number_priority_hd';
    const SORT_CH_BY_TITLE_PRIORITY_HD_FUNC   = 'SortingDialog::sort_ch_by_title_priority_hd';
    
    private $_owner;
	private $ch_sorting_method_idx;
	private $hd_ch_sorting_method_idx;   	

	////////////////////////////////////////////////////////////////////////////

    public function __construct($owner)
    {
        $this->_owner = $owner;
        $this->ch_sorting_method_idx = 0;
        $this->hd_ch_sorting_method_idx = 0;        

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

	public function get_handler_id()
	{
		return self::ID.'_handler';
	}

    protected function do_get_controls_defs(&$plugin_cookies)
	{
        //hd_print("plugin_cookies: ".json_encode($plugin_cookies));
        $defs = array();
        
        ControlFactory::add_combobox($defs, $this, null, 'ch_sorting_combobox', T::t('cap_channels_sort'),
                                     $this->ch_sorting_method_idx, array(
                                        '0' => T::t('cap_sort_by_number'),
                                        '1' => T::t('cap_sort_by_title')), 0, true);
        ControlFactory::add_combobox($defs, $this, null, 'hd_ch_sorting_combobox', T::t('cap_hd_channels_sorting'),
                                     $this->hd_ch_sorting_method_idx, array(
                                        '0' => T::t('cap_hd_sort_by_default'),
                                        '2' => T::t('cap_hd_sort_on_top_list')), 0, true);
        ControlFactory::add_vgap($defs, 40);        

        $add_params = array('button_caption_centered' => 1);
        ControlFactory::add_close_dialog_and_apply_button($defs, $this, $add_params, 'sort_apply_btn', T::t('cap_btn_apply'), 300);        

        return $defs;
    }    

	public function sort_channels(&$plugin_cookies)
	{
		if (!isset($plugin_cookies->{self::SORTING_METHOD_ATTR_NAME}))
			$plugin_cookies->{self::SORTING_METHOD_ATTR_NAME} = ($this->ch_sorting_method_idx + $this->hd_ch_sorting_method_idx);

		switch ($plugin_cookies->{self::SORTING_METHOD_ATTR_NAME})
		{
			case '0':
				$this->_owner->tv->channels->usort(self::SORT_CH_BY_NUMBER_FUNC);
				break;
			case '1':
				$this->_owner->tv->channels->usort(self::SORT_CH_BY_TITLE_FUNC);
				break;
			case '2':
				$this->_owner->tv->channels->usort(self::SORT_CH_BY_NUMBER_PRIORITY_HD_FUNC);
				break;
			case '3':
				$this->_owner->tv->channels->usort(self::SORT_CH_BY_TITLE_PRIORITY_HD_FUNC);
		}		
	}
    
    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        /* ---------- BEGIN DEBUG ---------- */
        //hd_print('SortingDialog $user_input =');
        //foreach ($user_input as $key => $value)
        //    hd_print("  $key => $value");
        /* ----------- END DEBUG ----------- */

        $control_id = $user_input->control_id;

        if ($user_input->action_type == 'confirm')
            {
                $new_value = $user_input->{$control_id};

                if ($control_id == 'ch_sorting_combobox'):
                    $this->ch_sorting_method_idx = $new_value;
                elseif ($control_id == 'hd_ch_sorting_combobox'):
                        $this->hd_ch_sorting_method_idx = $new_value;
                endif;
            }
        else if ($user_input->action_type == 'apply')
                if ($control_id == 'sort_apply_btn')
                    {
                        $plugin_cookies->{self::SORTING_METHOD_ATTR_NAME} = ($this->ch_sorting_method_idx + $this->hd_ch_sorting_method_idx);
        
                        $this->sort_channels($plugin_cookies);
                        
                        $parent_media_url = MediaURL::decode($user_input->parent_media_url);
                        $range = $this->_owner->get_folder_range($parent_media_url, 0, $plugin_cookies);
                        return ActionFactory::update_regular_folder($range, true, $user_input->sel_ndx);
                    }

        return ActionFactory::reset_controls($this->do_get_controls_defs($plugin_cookies));
    }

    public function show_sorting_dialog(&$plugin_cookies)
    {
        if (isset($plugin_cookies->{self::SORTING_METHOD_ATTR_NAME}))
            switch ($plugin_cookies->{self::SORTING_METHOD_ATTR_NAME})
            {
                case '0':
                    $this->ch_sorting_method_idx = 0;
                    $this->hd_ch_sorting_method_idx = 0;
                    break;
                case '1':
                    $this->ch_sorting_method_idx = 1;
                    $this->hd_ch_sorting_method_idx = 0;
                    break;
                case '2':
                    $this->ch_sorting_method_idx = 0;
                    $this->hd_ch_sorting_method_idx = 2;
                    break;
                case '3':
                    $this->ch_sorting_method_idx = 1;
                    $this->hd_ch_sorting_method_idx = 2;                
            }

        return ActionFactory::show_dialog(T::t('title_channels_sorting'), $this->do_get_controls_defs($plugin_cookies), true);
    }
    
    ////////////////////////////////////////////////////////////////////////////    
}

////////////////////////////////////////////////////////////////////////////////
?>
