<?php
////////////////////////////////////////////////////////////////////////////////

require_once 'lib/tv/tv_channel_list_screen.php';

////////////////////////////////////////////////////////////////////////////////

class TvChannelsScreen extends TvChannelListScreen
{

	public $tv;
	public static $ch_sorter;

	////////////////////////////////////////////////////////////////////////////

	public function __construct($tv)
	{
		parent::__construct($tv);

		$this->tv = $tv;

		self::$ch_sorter = SortingDialog::init($this);
		UserInputHandlerRegistry::get_instance()->register_handler($this);
	}

	////////////////////////////////////////////////////////////////////////////

	public function get_action_map(MediaURL $media_url, &$plugin_cookies)
	{
		/* ---------- BEGIN DEBUG ---------- */
		//hd_print(sprintf('TvChannelsScreen get_action_map $media_url = %s', json_encode($media_url)));
		/* ----------- END DEBUG ----------- */

		$actions = parent::get_action_map($media_url, $plugin_cookies);

		if ($this->tv->is_favorites_supported())
		{
			$add_favorite_action = UserInputHandlerRegistry::create_action($this, 'add_favorite');
			$add_favorite_action['caption'] = T::t('cap_add_to_fav_group');
			//$actions[GUI_EVENT_KEY_D_BLUE] = $add_favorite_action;
		}

		$show_sorter_action = UserInputHandlerRegistry::create_action($this, 'show_sorter');
		$show_sorter_action['caption'] = T::t('cap_channels_sorting');
		$actions[GUI_EVENT_KEY_C_YELLOW] = $show_sorter_action;

		return $actions;
	}

	public function handle_user_input(&$user_input, &$plugin_cookies)
	{
		/******* BEGIN DEBUG *******
		hd_print('TvChannelsScreen $user_input =');
		foreach ($user_input as $key => $value)
			hd_print("  $key => $value");
		// Return array:
		handler_id => tv_channel_list
		control_id => info
		parent_media_url => {"screen_id":"tv_channel_list","group_id":"4"}
		selected_media_url => {"channel_id":"458","group_id":"4"}
		sel_ndx => 0
		******* END DEBUG *******/

		if (!isset($user_input->selected_media_url))
			return null;

		$parent_media_url = MediaURL::decode($user_input->parent_media_url);
		$selected_media_url = MediaURL::decode($user_input->selected_media_url);
		$channel_id = $selected_media_url->channel_id;
		$is_favorite = $this->tv->is_favorite_channel_id($channel_id, $plugin_cookies);

		switch ($user_input->control_id)
		{
			case 'info':
				$c = $this->tv->get_channel($channel_id);
				//$id = $c->get_id();
				//$title = $c->get_title();
				//$number = $c->get_number();
				//$desc = $c->get_desc();
				//$lang = $c->get_lang();
				$dbg_info['cid'] = $channel_id;
				$dbg_info['epg_id'] = stristr($channel_id, '_', true);
				$dbg_info['title'] = $c->get_title();
				$dbg_info['number'] = $c->get_number();
				$dbg_info['icon'] = $c->get_icon_url();
				$dbg_info['url'] = $c->get_streaming_url();
				return Messages::show_tv_channel_debug_msg($dbg_info);
				//Messages::show_tv_channel_info_msg($number.". ".$title, $desc, $lang);

			case 'popup_menu':
				if (!$is_favorite)
				{
					$add_favorite_action = UserInputHandlerRegistry::create_action($this, 'add_favorite');
					$menu_items[] = array(
						GuiMenuItemDef::caption => T::t('cap_add_to_fav_list'),
						GuiMenuItemDef::action => $add_favorite_action);
				}
				else
				{
					$remove_favorite_action = UserInputHandlerRegistry::create_action($this, 'remove_favorite');
					$menu_items[] = array(
						GuiMenuItemDef::caption => T::t('cap_remove_from_fav_list'),
						GuiMenuItemDef::action => $remove_favorite_action);
				}
				$add_recycle_bin_action = UserInputHandlerRegistry::create_action($this, 'move_to_recycle_bin');
				$menu_items[] = array(
					GuiMenuItemDef::caption => T::t('cap_remove_to_recycle_bin'),
					GuiMenuItemDef::action => $add_recycle_bin_action);
				return ActionFactory::show_popup_menu($menu_items);

			case 'show_sorter':
				return self::$ch_sorter->show_sorting_dialog($plugin_cookies);

			case 'add_favorite':
				if (!$is_favorite)
				{
					$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_ADD, $channel_id, $plugin_cookies);
					return $this->get_sel_item_update_action($user_input, $plugin_cookies);
				}

			case 'remove_favorite':
				if ($is_favorite)
				{
					$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_REMOVE, $channel_id, $plugin_cookies);
					return $this->get_sel_item_update_action($user_input, $plugin_cookies);
				}

			case 'move_to_recycle_bin':
				$exclude_channel_ids = $this->tv->get_exclude_channel_ids($plugin_cookies);
				array_push($exclude_channel_ids, $channel_id);
				TvRecycleBinScreen::set_channel_ids($plugin_cookies, $exclude_channel_ids);

				$sel_ndx = $user_input->sel_ndx - 1;
				if ($sel_ndx < 0)
					$sel_ndx = 0;

				$range = $this->get_folder_range($parent_media_url, 0, $plugin_cookies);
				return ActionFactory::update_regular_folder($range, true, $sel_ndx);
		}

		return null;
	}

	////////////////////////////////////////////////////////////////////////////

	public function get_handler_id()
	{
		return TvChannelListScreen::ID.'_handler';
	}

	public function get_sel_item_update_action(&$user_input, &$plugin_cookies)
	{
		$parent_media_url = MediaURL::decode($user_input->parent_media_url);
		$sel_ndx = $user_input->sel_ndx;
		$group = $this->tv->get_group($parent_media_url->group_id);
		$channel_ids = $group->get_channels($plugin_cookies);
		$ch = $this->tv->get_channel($channel_ids[$sel_ndx]);

		$items[] = $this->get_regular_folder_item($group, $ch, $plugin_cookies);
		$range = HD::create_regular_folder_range($items, $sel_ndx, count($channel_ids));

		return ActionFactory::update_regular_folder($range, false);
	}

	public function get_regular_folder_item($group, $c, &$plugin_cookies)
	{
		$title = $c->get_title();
		$desc = $c->get_desc();
		$lang = $c->get_lang();

		return array
		(
			PluginRegularFolderItem::media_url =>
				MediaURL::encode(
					array(
						'channel_id' => $c->get_id(),
						'group_id' => $group->get_id())),
			PluginRegularFolderItem::caption => $title,
			PluginRegularFolderItem::view_item_params => array
			(
				ViewItemParams::icon_path => $c->get_icon_url(),
				ViewItemParams::item_detailed_icon_path => $c->get_icon_url(),
				ViewItemParams::item_detailed_info => "Название:|$title||Язык вещания:|$lang||Описание:|$desc",
			),
			PluginRegularFolderItem::starred => $this->tv->is_favorite_channel_id($c->get_id(), $plugin_cookies),
		);
	}

	public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
	{
		$this->tv->folder_entered($media_url, $plugin_cookies);
		$this->tv->ensure_channels_loaded($plugin_cookies);

		try
		{
			$group = $this->tv->get_group($media_url->group_id);
		}
		catch (Exception $e)
		{
			throw new Exception("Warning ".$this->get_handler_id().": group ".$media_url->group_id." not found.");
			return null;
		}

		$ch_ids_arr = $group->get_channels($plugin_cookies);
		$items = array();

		foreach ($ch_ids_arr as $cid)
		{
			$ch = $this->tv->get_channel(strval($cid));
			$items[] = $this->get_regular_folder_item($group, $ch, $plugin_cookies);
		}

		return $items;
	}

	///////////////////////////////////////////////////////////////////////////

}

////////////////////////////////////////////////////////////////////////////////
?>
