<?php
////////////////////////////////////////////////////////////////////////////////

require_once 'lib/tv/tv_favorites_screen.php';

////////////////////////////////////////////////////////////////////////////////

class TvFavChannelsScreen extends TvFavoritesScreen
{
	protected $tv;

	public function __construct(Tv $tv)
	{
		$this->tv = $tv;

		parent::__construct($tv);
	}

	protected function do_get_folder_views(&$plugin_cookies)
	{
		$views = parent::do_get_folder_views($plugin_cookies);
		$views[0][PluginRegularFolderView::view_params][ViewParams::background_path] = 'plugin_file://img/favorites_bg.png';
		$views[1][PluginRegularFolderView::view_params][ViewParams::background_path] = 'plugin_file://img/favorites_bg.png';
		return $views;
	} 

	protected function do_update_action($sel_increment, &$user_input, &$plugin_cookies) {
		$parent_media_url = MediaURL::decode($user_input->parent_media_url);
		$num_favorites = count($this->tv->get_fav_channel_ids($plugin_cookies));
		$sel_ndx = $user_input->sel_ndx + $sel_increment;
		if ($sel_ndx < 0)
			$sel_ndx = 0;
		if ($sel_ndx >= $num_favorites)
			$sel_ndx = $num_favorites - 1;

		$range = HD::create_regular_folder_range($this->get_all_folder_items($parent_media_url, $plugin_cookies));
		return ActionFactory::update_regular_folder($range, true, $sel_ndx);
	}	
	
	public function get_action_map(MediaURL $media_url, &$plugin_cookies)
	{
		$move_backward_favorite_action =
			UserInputHandlerRegistry::create_action($this, 'move_backward_favorite');
		$move_backward_favorite_action['caption'] = T::t('cap_fav_channel_backward');

		$move_forward_favorite_action =
			UserInputHandlerRegistry::create_action($this, 'move_forward_favorite');
		$move_forward_favorite_action['caption'] = T::t('cap_fav_channel_forward');

		$remove_favorite_action =
			UserInputHandlerRegistry::create_action($this, 'remove_favorite');
		$remove_favorite_action['caption'] = T::t('cap_fav_channel_remove');

		$clear_favorite_action =
			UserInputHandlerRegistry::create_action($this, 'clear_favorite');
		$remove_favorite_action['caption'] = T::t('cap_fav_list_clear');        
		
		$menu_items[] = array(
			GuiMenuItemDef::caption => T::t('cap_remove_from_fav_list'),
			GuiMenuItemDef::action => $remove_favorite_action);
		$menu_items[] = array(
			GuiMenuItemDef::is_separator => true);
		$menu_items[] = array(
			GuiMenuItemDef::caption => T::t('cap_fav_list_clear'),
			GuiMenuItemDef::action => $clear_favorite_action);
		
		$popup_menu_action = ActionFactory::show_popup_menu($menu_items);

		return array
		(
			GUI_EVENT_KEY_ENTER => ActionFactory::tv_play(),
			GUI_EVENT_KEY_PLAY  => ActionFactory::tv_play(),
			GUI_EVENT_KEY_B_GREEN => $move_backward_favorite_action,
			GUI_EVENT_KEY_C_YELLOW => $move_forward_favorite_action,
			//GUI_EVENT_KEY_D_BLUE => $remove_favorite_action,
			GUI_EVENT_KEY_POPUP_MENU => $popup_menu_action,
		);
	}
 
	public function handle_user_input(&$user_input, &$plugin_cookies) {

		if (!isset($user_input->selected_media_url))
			return null;

		$media_url = MediaURL::decode($user_input->selected_media_url);
		$channel_id = $media_url->channel_id;		
		
		if ($user_input->control_id == 'move_backward_favorite') {
			$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_MOVE_DOWN, $channel_id, $plugin_cookies);
			return $this->do_update_action(1, $user_input, $plugin_cookies);
		} else		
		if ($user_input->control_id == 'move_forward_favorite') {
			$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_MOVE_UP, $channel_id, $plugin_cookies);
			return $this->do_update_action(-1, $user_input, $plugin_cookies);
		} else	
		if ($user_input->control_id == 'remove_favorite') {
			$this->tv->change_tv_favorites(PLUGIN_FAVORITES_OP_REMOVE, $channel_id, $plugin_cookies);
			return $this->do_update_action(0, $user_input, $plugin_cookies);
		} else	
		if ($user_input->control_id == 'clear_favorite') {
			$plugin_cookies->favorite_channels = '';
			return $this->do_update_action(0, $user_input, $plugin_cookies);
		}

		return null;
	}
 
	public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
	{
		$this->tv->folder_entered($media_url, $plugin_cookies);
		
		try
		{
			$group = $this->tv->get_group('__favorites');
		}
		catch (Exception $e)
		{
			throw new Exception("Warning ".$this->get_handler_id().": group ".$media_url->group_id." not found.");
			return null;
		}        
		
		$items = array();
		
		$fav_channel_ids = $group->get_channels($plugin_cookies); 

		foreach ($fav_channel_ids as $cid)
		{
			if (preg_match('/^\s*$/', $cid))
				continue;

			try
			{
				$c = $this->tv->get_channel($cid);
			}
			catch (Exception $e)
			{
				hd_print("Warning: channel '$cid' not found.");
				continue;
			}

			array_push($items,
				array
				(
					PluginRegularFolderItem::media_url =>
						MediaURL::encode(
							array(
								'channel_id' => $c->get_id(),
								'group_id' => '__favorites')),
					PluginRegularFolderItem::caption => $c->get_title(),
					PluginRegularFolderItem::view_item_params => array
					(
						ViewItemParams::icon_path => $c->get_icon_url(),
						ViewItemParams::item_detailed_icon_path => $c->get_icon_url(),
					),
					PluginRegularFolderItem::starred => false,
				));
		}

		return $items;
	}    
}

?>
