<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/tv/tv_group_list_screen.php';
require_once 'lib/action_factory.php';

///////////////////////////////////////////////////////////////////////////

class TvGroupsScreen extends TvGroupListScreen
	implements UserInputHandler
{

	public static function get_media_url_str($group_id)
	{
		return MediaURL::encode(
			array
			(
				'screen_id' => TvGroupListScreen::ID,
				'group_id'  => $group_id,
			));
	}

	///////////////////////////////////////////////////////////////////////

	protected $settings;

	///////////////////////////////////////////////////////////////////////

	public function __construct($tv)
	{
		parent::__construct($tv);
	
		$this->settings = SettingsDialog::init($this);
		$this->settings->tv = $tv;
		
		UserInputHandlerRegistry::get_instance()->register_handler($this);
	}

	///////////////////////////////////////////////////////////////////////

	public function get_action_map(MediaURL $media_url, &$plugin_cookies)
	{
		$actions = array();
		$actions = parent::get_action_map($media_url, &$plugin_cookies);

		$open_setup_action = UserInputHandlerRegistry::create_action($this, 'open_setup');
		$open_setup_action['caption'] = T::t('cap_settings');
		$actions[GUI_EVENT_KEY_B_GREEN] = $open_setup_action;

		//$open_recycle_bin_action = ActionFactory::open_folder(TvRecycleBinScreen::get_media_url_str());
		//$open_recycle_bin_action['caption'] = T::t('title_recycle_bin');               
		//$actions[GUI_EVENT_KEY_C_YELLOW] = $open_recycle_bin_action;
		
		$tv_play_action = UserInputHandlerRegistry::create_action($this, 'tv_play');
		$actions[GUI_EVENT_KEY_PLAY] = $tv_play_action;

	return $actions;
	}

	///////////////////////////////////////////////////////////////////////

	public function get_handler_id()
	{
		return TvGroupListScreen::ID.'_handler';
	}

	///////////////////////////////////////////////////////////////////////

	public function handle_user_input(&$user_input, &$plugin_cookies)
	{
		/******* BEGIN DEBUG ******* 
		hd_print(sprintf('TvGroupsScreen handle_user_input $input = %s', json_encode($user_input)));
		hd_print('TvGroupsScreen => $user_input:');
		foreach ($user_input as $key => $value)
			hd_print("  $key => $value");
		// Return array:
		handler_id => tv_group_list,
		control_id => shortcut,
		action_type => confirm,
		parent_media_url => tv_group_list,
		selected_media_url => {"screen_id":"tv_channel_list","group_id":"4"},
		sel_ndx => 6,
		shortcut => yes,
		selected_control_id => shortcut,
		******* END DEBUG *******/

		if ($user_input->control_id == 'open_setup')
			return $this->settings->show_settings_dialog($plugin_cookies);
		else if ($user_input->control_id == 'tv_play')
				if (MediaURL::decode($user_input->selected_media_url)->screen_id !== TvRecycleBinScreen::ID)
					return ActionFactory::tv_play();

		return null;
	}
	
	public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
	{
		$items = parent::get_all_folder_items($media_url, $plugin_cookies);

		$items[0][PluginRegularFolderItem::view_item_params][ViewItemParams::item_caption_color] = 21;

		return $items;
	} 
	
}

///////////////////////////////////////////////////////////////////////////
?>
