<?php

require_once 'lib/tv/tv.php';
require_once 'lib/abstract_preloaded_regular_screen.php';

class TvRecycleBinScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'tv_recycle_bin';

    public static function get_media_url_str()
    {
        return MediaURL::encode(
            array(
                'screen_id' => self::ID));
    }

    ///////////////////////////////////////////////////////////////////////

    private $tv;

    public function __construct(Tv $tv)
    {
        $this->tv = $tv;

        parent::__construct(self::ID);

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    protected function do_get_folder_views(&$plugin_cookies)
    {
        return $this->tv->get_tv_channel_list_folder_views($plugin_cookies);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $recycle_bin_restore_action = UserInputHandlerRegistry::create_action($this, 'recycle_bin_restore');
        $recycle_bin_restore_action['caption'] = T::t('cap_restore_from_recycle_bin');
        
        $clear_recycle_bin_action = UserInputHandlerRegistry::create_action($this, 'recycle_bin_clear');
        $clear_recycle_bin_action['caption'] = T::t('cap_recycle_bin_clear');
       
        $menu_items[] = array(
            GuiMenuItemDef::caption => T::t('cap_restore_from_recycle_bin'),
            GuiMenuItemDef::action => $recycle_bin_restore_action);

        $popup_menu_action = ActionFactory::show_popup_menu($menu_items);

        return array
        (
            GUI_EVENT_KEY_C_YELLOW => $clear_recycle_bin_action,
            GUI_EVENT_KEY_POPUP_MENU => $popup_menu_action,
        );
    }

    public function get_handler_id()
    { return self::ID.'_handler'; }

    public static function set_channel_ids(&$plugin_cookies, &$ids)
    {
        $plugin_cookies->{'excluded_channels'} = join(',', $ids);
    }

    private function get_update_action($sel_increment, &$user_input, &$plugin_cookies)
    {
        $parent_media_url = MediaURL::decode($user_input->parent_media_url);        
        $num_items = count($this->tv->get_exclude_channel_ids($plugin_cookies));
        
        $sel_ndx = $user_input->sel_ndx + $sel_increment;
        if ($sel_ndx < 0)
            $sel_ndx = 0;
        if ($sel_ndx >= $num_items)
            $sel_ndx = $num_items - 1;

        $range = HD::create_regular_folder_range(
            $this->get_all_folder_items(
                $parent_media_url, $plugin_cookies));

        return ActionFactory::update_regular_folder($range, true, $sel_ndx);
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        /******* BEGIN DEBUG *******
        hd_print('recycle_bin_channels::handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");
        ******* END DEBUG *******/

        $parent_media_url = MediaURL::decode($user_input->parent_media_url);
        $selected_media_url = MediaURL::decode($user_input->selected_media_url);
        $channel_id = $selected_media_url->channel_id;
        $sel_ndx = 0;
        
        if ($user_input->control_id == 'recycle_bin_restore')
        {
            if (!isset($user_input->selected_media_url))
                return null;

            $exclude_channel_ids = $this->tv->get_exclude_channel_ids($plugin_cookies);
            $key = array_search($channel_id, $exclude_channel_ids);
            if ($key !== false)
                unset($exclude_channel_ids[$key]);                
               
            $num_items = count($exclude_channel_ids);
    
            $sel_ndx = $user_input->sel_ndx - 1;
            if ($sel_ndx < 0)
                $sel_ndx = 0;
            if ($sel_ndx >= $num_items)
                $sel_ndx = $num_items - 1;

            $this->get_update_action(-1, $user_input, $plugin_cookies);
        }
        else if ($user_input->control_id == 'recycle_bin_clear')
                $exclude_channel_ids = array();
       
        self::set_channel_ids($plugin_cookies, $exclude_channel_ids);

        $range = HD::create_regular_folder_range($this->get_all_folder_items($parent_media_url, $plugin_cookies));        
        return ActionFactory::update_regular_folder($range, true, $sel_ndx);
    }

    
    
    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->tv->folder_entered($media_url, $plugin_cookies);
        
        $exclude_channel_ids = $this->tv->get_exclude_channel_ids($plugin_cookies);
        $items = array();

        foreach ($exclude_channel_ids as $cid)
        {
            if (preg_match('/^\s*$/', $cid))
                continue;

            try
            {
                $channel = $this->tv->get_channel($cid);
            }
            catch (Exception $e)
            {
                hd_print("Warning: channel '$cid' not found.");
                
                $key = array_search($cid, $exclude_channel_ids);
                if ($key !== false)
                    unset($exclude_channel_ids[$key]);
                    
                self::set_channel_ids($plugin_cookies, $exclude_channel_ids);
                
                continue;
            }

            array_push($items,
                array
                (
                    PluginRegularFolderItem::media_url =>
                        MediaURL::encode(
                            array(
                                'channel_id' => $channel->get_id(),
                                'group_id' => '__recycle_bin')),
                    PluginRegularFolderItem::caption => $channel->get_title(),
                    PluginRegularFolderItem::view_item_params => array
                    (
                        ViewItemParams::icon_path => $channel->get_icon_url(),
                        ViewItemParams::item_detailed_icon_path => $channel->get_icon_url(),
                    ),
                    PluginRegularFolderItem::starred => false,
                ));
        }

        return $items;
    }

    public function get_archive(MediaURL $media_url, &$plugin_cookies)
    {
        return $this->tv->get_archive($media_url, $plugin_cookies);
    }
}

?>
